// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

mixin LoggingApi on _SDKLoggerMixin {

  Future<T> wrapper<T>(String apiName, Future<T> apiAction, [Map? args]) async {
    if (args != null && args.isNotEmpty) {
      apiLogger.i('$apiName, args=$args');
    } else {
      apiLogger.i(apiName);
    }
    final start = DateTime.now();
    return apiAction.then((value) {
      final elapsed = DateTime.now().difference(start);
      dynamic result;
      if (value is IntValue) {
        result = value.value;
      } else if (value is BoolValue) {
        result = value.value;
      } else if (value is DoubleValue) {
        result = value.value;
      }
      apiLogger.i('$apiName done. result=$result, elapsed=${elapsed.inMilliseconds}ms');
      return value;
    });
  }

}


class _LoggingEngineApi with _SDKLoggerMixin, LoggingApi implements EngineApi {

  final _api = EngineApi();

  @override
  String get logTag => 'EngineApi';

  @override
  Future<IntValue> addLiveStreamTask(AddOrUpdateLiveStreamTaskRequest arg) {
    return wrapper('addLiveStreamTask', _api.addLiveStreamTask(arg));
  }

  @override
  Future<IntValue> adjustPlaybackSignalVolume(IntValue arg) {
    return wrapper('adjustPlaybackSignalVolume', _api.adjustPlaybackSignalVolume(arg));
  }

  @override
  Future<IntValue> adjustRecordingSignalVolume(IntValue arg) {
    return wrapper('adjustRecordingSignalVolume', _api.adjustRecordingSignalVolume(arg));
  }

  @override
  Future<IntValue> adjustUserPlaybackSignalVolume(AdjustUserPlaybackSignalVolumeRequest arg) {
    return wrapper('adjustUserPlaybackSignalVolume', _api.adjustUserPlaybackSignalVolume(arg));
  }

  @override
  Future<IntValue> clearStatsEventCallback() {
    return wrapper('clearStatsEventCallback', _api.clearStatsEventCallback());
  }

  @override
  Future<IntValue> create(CreateEngineRequest arg) {
    return wrapper('create', _api.create(arg));
  }

  @override
  Future<IntValue> enableAudioVolumeIndication(EnableAudioVolumeIndicationRequest arg) {
    return wrapper('enableAudioVolumeIndication', _api.enableAudioVolumeIndication(arg));
  }

  @override
  Future<IntValue> enableDualStreamMode(BoolValue arg) {
    return wrapper('enableDualStreamMode', _api.enableDualStreamMode(arg));
  }

  @override
  Future<IntValue> enableEncryption(EnableEncryptionRequest arg) {
    return wrapper('enableEncryption', _api.enableEncryption(arg));
  }

  @override
  Future<IntValue> enableLocalAudio(BoolValue arg) {
    return wrapper('enableLocalAudio', _api.enableLocalAudio(arg), {'enable': arg.value});
  }

  @override
  Future<IntValue> enableLocalVideo(BoolValue arg) {
    return wrapper('enableLocalVideo', _api.enableLocalVideo(arg), {'enable': arg.value});
  }

  @override
  Future<IntValue> enableSuperResolution(BoolValue arg) {
    return wrapper('enableSuperResolution', _api.enableSuperResolution(arg), {'enable': arg.value});
  }

  @override
  Future<IntValue> getConnectionState() {
    return wrapper('getConnectionState', _api.getConnectionState());
  }

  @override
  Future<IntValue> joinChannel(JoinChannelRequest arg) {
    return wrapper('joinChannel', _api.joinChannel(arg), {'channel': arg.channelName, 'uid': arg.uid});
  }

  @override
  Future<IntValue> leaveChannel() {
    return wrapper('leaveChannel', _api.leaveChannel());
  }

  @override
  Future<IntValue> muteLocalAudioStream(BoolValue arg) {
    return wrapper('muteLocalAudioStream', _api.muteLocalAudioStream(arg), {'mute': arg.value});
  }

  @override
  Future<IntValue> muteLocalVideoStream(BoolValue arg) {
    return wrapper('muteLocalVideoStream', _api.muteLocalVideoStream(arg), {'mute': arg.value});
  }

  @override
  Future<IntValue> release() {
    return wrapper('release', _api.release());
  }

  @override
  Future<IntValue> removeLiveStreamTask(DeleteLiveStreamTaskRequest arg) {
    return wrapper('removeLiveStreamTask', _api.removeLiveStreamTask(arg));
  }

  @override
  Future<IntValue> sendSEIMsg(SendSEIMsgRequest arg) {
    return wrapper('sendSEIMsg', _api.sendSEIMsg(arg));
  }

  @override
  Future<IntValue> setAudioEffectPreset(IntValue arg) {
    return wrapper('setAudioEffectPreset', _api.setAudioEffectPreset(arg));
  }

  @override
  Future<IntValue> setAudioProfile(SetAudioProfileRequest arg) {
    return wrapper('setAudioProfile', _api.setAudioProfile(arg), {'profile': arg.profile, 'scenario': arg.scenario});
  }

  @override
  Future<IntValue> setCameraCaptureConfig(SetCameraCaptureConfigRequest arg) {
    return wrapper('setCameraCaptureConfig', _api.setCameraCaptureConfig(arg));
  }

  @override
  Future<IntValue> setChannelProfile(IntValue arg) {
    return wrapper('setChannelProfile', _api.setChannelProfile(arg));
  }

  @override
  Future<IntValue> setClientRole(IntValue arg) {
    return wrapper('setClientRole', _api.setClientRole(arg));
  }

  @override
  Future<IntValue> setLocalMediaPriority(SetLocalMediaPriorityRequest arg) {
    return wrapper('setLocalMediaPriority', _api.setLocalMediaPriority(arg));
  }

  @override
  Future<IntValue> setLocalPublishFallbackOption(IntValue arg) {
    return wrapper('setLocalPublishFallbackOption', _api.setLocalPublishFallbackOption(arg));
  }

  @override
  Future<IntValue> setLocalVideoConfig(SetLocalVideoConfigRequest arg) {
    return wrapper('setLocalVideoConfig', _api.setLocalVideoConfig(arg));
  }

  @override
  Future<IntValue> setLocalVoiceEqualization(SetLocalVoiceEqualizationRequest arg) {
    return wrapper('setLocalVoiceEqualization', _api.setLocalVoiceEqualization(arg));
  }

  @override
  Future<IntValue> setLocalVoicePitch(DoubleValue arg) {
    return wrapper('setLocalVoicePitch', _api.setLocalVoicePitch(arg));
  }

  @override
  Future<IntValue> setRemoteSubscribeFallbackOption(IntValue arg) {
    return wrapper('setRemoteSubscribeFallbackOption', _api.setRemoteSubscribeFallbackOption(arg));
  }

  @override
  Future<IntValue> setStatsEventCallback() {
    return wrapper('setStatsEventCallback', _api.setStatsEventCallback());
  }

  @override
  Future<IntValue> setVoiceBeautifierPreset(IntValue arg) {
    return wrapper('setVoiceBeautifierPreset', _api.setVoiceBeautifierPreset(arg));
  }

  @override
  Future<IntValue> startAudioDump() {
    return wrapper('startAudioDump', _api.startAudioDump());
  }

  @override
  Future<IntValue> startAudioRecording(StartAudioRecordingRequest arg) {
    return wrapper('startAudioRecording', _api.startAudioRecording(arg));
  }

  @override
  Future<IntValue> startChannelMediaReplay(StartOrUpdateChannelMediaReplayRequest arg) {
    return wrapper('startChannelMediaReplay', _api.startChannelMediaReplay(arg));
  }

  @override
  Future<IntValue> startScreenCapture(StartScreenCaptureRequest arg) {
    return wrapper('startScreenCapture', _api.startScreenCapture(arg));
  }

  @override
  Future<IntValue> startVideoPreview() {
    return wrapper('startVideoPreview', _api.startVideoPreview());
  }

  @override
  Future<IntValue> stopAudioDump() {
    return wrapper('stopAudioDump', _api.stopAudioDump());
  }

  @override
  Future<IntValue> stopAudioRecording() {
    return wrapper('stopAudioRecording', _api.stopAudioRecording());
  }

  @override
  Future<IntValue> stopChannelMediaRelay() {
    return wrapper('stopChannelMediaRelay', _api.stopChannelMediaRelay());
  }

  @override
  Future<IntValue> stopScreenCapture() {
    return wrapper('stopScreenCapture', _api.stopScreenCapture());
  }

  @override
  Future<IntValue> stopVideoPreview() {
    return wrapper('stopVideoPreview', _api.stopVideoPreview());
  }

  @override
  Future<IntValue> subscribeAllRemoteAudio(BoolValue arg) {
    return wrapper('subscribeAllRemoteAudio', _api.subscribeAllRemoteAudio(arg), {'subscribe': arg.value});
  }

  @override
  Future<IntValue> subscribeRemoteAudio(SubscribeRemoteAudioRequest arg) {
    return wrapper('subscribeRemoteAudio', _api.subscribeRemoteAudio(arg), {'uid': arg.uid, 'subscribe': arg.subscribe});
  }

  @override
  Future<IntValue> subscribeRemoteSubStreamVideo(SubscribeRemoteSubStreamVideoRequest arg) {
    return wrapper('subscribeRemoteSubStreamVideo', _api.subscribeRemoteSubStreamVideo(arg), {'uid': arg.uid, 'subscribe': arg.subscribe});
  }

  @override
  Future<IntValue> subscribeRemoteVideo(SubscribeRemoteVideoRequest arg) {
    return wrapper('subscribeRemoteVideo', _api.subscribeRemoteVideo(arg), {'uid': arg.uid, 'subscribe': arg.subscribe});
  }

  @override
  Future<IntValue> switchChannel(SwitchChannelRequest arg) {
    return wrapper('switchChannel', _api.switchChannel(arg), {'channelName': arg.channelName});
  }

  @override
  Future<IntValue> updateChannelMediaRelay(StartOrUpdateChannelMediaReplayRequest arg) {
    return wrapper('updateChannelMediaRelay', _api.updateChannelMediaRelay(arg));
  }

  @override
  Future<IntValue> updateLiveStreamTask(AddOrUpdateLiveStreamTaskRequest arg) {
    return wrapper('updateLiveStreamTask', _api.updateLiveStreamTask(arg));
  }

  @override
  Future<IntValue> uploadSdkInfo() {
    return wrapper('uploadSdkInfo', _api.uploadSdkInfo());
  }

}

class _LoggingVideoRendererApi with _SDKLoggerMixin, LoggingApi implements VideoRendererApi {

  final _api = VideoRendererApi();

  @override
  String get logTag => 'VideoRendererApi';

  @override
  Future<IntValue> createVideoRenderer() {
    return wrapper('createVideoRenderer', _api.createVideoRenderer());
  }

  @override
  Future<void> disposeVideoRenderer(IntValue arg) {
    return wrapper('disposeVideoRenderer', _api.disposeVideoRenderer(arg));
  }

  @override
  Future<IntValue> setMirror(SetVideoRendererMirrorRequest arg) {
    return wrapper('setMirror', _api.setMirror(arg));
  }

  @override
  Future<IntValue> setupLocalSubStreamVideoRenderer(IntValue arg) {
    return wrapper('setupLocalSubStreamVideoRenderer', _api.setupLocalSubStreamVideoRenderer(arg), {'textureId': arg.value});
  }

  @override
  Future<IntValue> setupLocalVideoRenderer(IntValue arg) {
    return wrapper('setupLocalVideoRenderer', _api.setupLocalVideoRenderer(arg), {'textureId': arg.value});
  }

  @override
  Future<IntValue> setupRemoteSubStreamVideoRenderer(SetupRemoteSubStreamVideoRendererRequest arg) {
    return wrapper('setupRemoteSubStreamVideoRenderer', _api.setupRemoteSubStreamVideoRenderer(arg), {'uid': arg.uid, 'textureId': arg.textureId});
  }

  @override
  Future<IntValue> setupRemoteVideoRenderer(SetupRemoteVideoRendererRequest arg) {
    return wrapper('setupRemoteVideoRenderer', _api.setupRemoteVideoRenderer(arg), {'uid': arg.uid, 'textureId': arg.textureId});
  }

}

class _LoggingDeviceManagerApi with _SDKLoggerMixin, LoggingApi implements DeviceManagerApi {

  final _api = DeviceManagerApi();

  @override
  String get logTag => 'DeviceManagerApi';

  @override
  Future<IntValue> clearDeviceEventCallback() {
    return wrapper('clearDeviceEventCallback', _api.clearDeviceEventCallback());
  }

  @override
  Future<IntValue> enableEarback(EnableEarbackRequest arg) {
    return wrapper('enableEarback', _api.enableEarback(arg), {'enable': arg.enabled, 'volume': arg.volume});
  }

  @override
  Future<DoubleValue> getCameraMaxZoom() {
    return wrapper('getCameraMaxZoom', _api.getCameraMaxZoom());
  }

  @override
  Future<BoolValue> isPlayoutDeviceMute() {
    return wrapper('isPlayoutDeviceMute', _api.isPlayoutDeviceMute());
  }

  @override
  Future<BoolValue> isRecordDeviceMute() {
    return wrapper('isRecordDeviceMute', _api.isRecordDeviceMute());
  }

  @override
  Future<BoolValue> isSpeakerphoneOn() {
    return wrapper('isSpeakerphoneOn', _api.isSpeakerphoneOn());
  }

  @override
  Future<IntValue> setAudioFocusMode(IntValue arg) {
    return wrapper('setAudioFocusMode', _api.setAudioFocusMode(arg), {'mode': arg.value});
  }

  @override
  Future<IntValue> setCameraFocusPosition(SetCameraFocusPositionRequest arg) {
    return wrapper('setCameraFocusPosition', _api.setCameraFocusPosition(arg));
  }

  @override
  Future<IntValue> setCameraTorchOn(BoolValue arg) {
    return wrapper('setCameraTorchOn', _api.setCameraTorchOn(arg));
  }

  @override
  Future<IntValue> setCameraZoomFactor(IntValue arg) {
    return wrapper('setCameraZoomFactor', _api.setCameraZoomFactor(arg));
  }

  @override
  Future<IntValue> setDeviceEventCallback() {
    return wrapper('setDeviceEventCallback', _api.setDeviceEventCallback());
  }

  @override
  Future<IntValue> setEarbackVolume(IntValue arg) {
    return wrapper('setEarbackVolume', _api.setEarbackVolume(arg));
  }

  @override
  Future<IntValue> setPlayoutDeviceMute(BoolValue arg) {
    return wrapper('setPlayoutDeviceMute', _api.setPlayoutDeviceMute(arg), {'mute': arg.value});
  }

  @override
  Future<IntValue> setRecordDeviceMute(BoolValue arg) {
    return wrapper('setRecordDeviceMute', _api.setRecordDeviceMute(arg), {'mute': arg.value});
  }

  @override
  Future<IntValue> setSpeakerphoneOn(BoolValue arg) {
    return wrapper('setSpeakerphoneOn', _api.setSpeakerphoneOn(arg), {'on': arg.value});
  }

  @override
  Future<IntValue> switchCamera() {
    return wrapper('switchCamera', _api.switchCamera());
  }

}

class _LoggingAudioMixingApi with _SDKLoggerMixin, LoggingApi implements AudioMixingApi {

  final _api = AudioMixingApi();

  @override
  String get logTag => 'AudioMixingApi';

  @override
  Future<IntValue> clearAudioMixingEventCallback() {
    return wrapper('clearAudioMixingEventCallback', _api.clearAudioMixingEventCallback());
  }

  @override
  Future<IntValue> getAudioMixingCurrentPosition() {
    return wrapper('getAudioMixingCurrentPosition', _api.getAudioMixingCurrentPosition());
  }

  @override
  Future<IntValue> getAudioMixingDuration() {
    return wrapper('getAudioMixingDuration', _api.getAudioMixingDuration());
  }

  @override
  Future<IntValue> getAudioMixingPlaybackVolume() {
    return wrapper('getAudioMixingPlaybackVolume', _api.getAudioMixingPlaybackVolume());
  }

  @override
  Future<IntValue> getAudioMixingSendVolume() {
    return wrapper('getAudioMixingSendVolume', _api.getAudioMixingSendVolume());
  }

  @override
  Future<IntValue> pauseAudioMixing() {
    return wrapper('pauseAudioMixing', _api.pauseAudioMixing());
  }

  @override
  Future<IntValue> resumeAudioMixing() {
    return wrapper('resumeAudioMixing', _api.resumeAudioMixing());
  }

  @override
  Future<IntValue> setAudioMixingEventCallback() {
    return wrapper('setAudioMixingEventCallback', _api.setAudioMixingEventCallback());
  }

  @override
  Future<IntValue> setAudioMixingPlaybackVolume(IntValue arg) {
    return wrapper('setAudioMixingPlaybackVolume', _api.setAudioMixingPlaybackVolume(arg), {'volume': arg.value});
  }

  @override
  Future<IntValue> setAudioMixingPosition(IntValue arg) {
    return wrapper('setAudioMixingPosition', _api.setAudioMixingPosition(arg), {'position': arg.value});
  }

  @override
  Future<IntValue> setAudioMixingSendVolume(IntValue arg) {
    return wrapper('setAudioMixingSendVolume', _api.setAudioMixingSendVolume(arg), {'volume': arg.value});
  }

  @override
  Future<IntValue> startAudioMixing(StartAudioMixingRequest arg) {
    return wrapper('startAudioMixing', _api.startAudioMixing(arg));
  }

  @override
  Future<IntValue> stopAudioMixing() {
    return wrapper('stopAudioMixing', _api.stopAudioMixing());
  }
}

class _LoggingAudioEffectApi with _SDKLoggerMixin, LoggingApi implements AudioEffectApi {

  final _api = AudioEffectApi();

  @override
  String get logTag => 'AudioEffectApi';

  @override
  Future<IntValue> clearAudioEffectEventCallback() {
    return wrapper('clearAudioEffectEventCallback', _api.clearAudioEffectEventCallback());
  }

  @override
  Future<IntValue> getEffectCurrentPosition(IntValue arg) {
    return wrapper('getEffectCurrentPosition', _api.getEffectCurrentPosition(arg));
  }

  @override
  Future<IntValue> getEffectDuration(IntValue arg) {
    return wrapper('getEffectDuration', _api.getEffectDuration(arg));
  }

  @override
  Future<IntValue> getEffectPlaybackVolume(IntValue arg) {
    return wrapper('getEffectPlaybackVolume', _api.getEffectPlaybackVolume(arg));
  }

  @override
  Future<IntValue> getEffectSendVolume(IntValue arg) {
    return wrapper('getEffectSendVolume', _api.getEffectSendVolume(arg));
  }

  @override
  Future<IntValue> pauseAllEffects() {
    return wrapper('pauseAllEffects', _api.pauseAllEffects());
  }

  @override
  Future<IntValue> pauseEffect(IntValue arg) {
    return wrapper('pauseEffect', _api.pauseEffect(arg));
  }

  @override
  Future<IntValue> playEffect(PlayEffectRequest arg) {
    return wrapper('playEffect', _api.playEffect(arg));
  }

  @override
  Future<IntValue> resumeAllEffects() {
    return wrapper('resumeAllEffects', _api.resumeAllEffects());
  }

  @override
  Future<IntValue> resumeEffect(IntValue arg) {
    return wrapper('resumeEffect', _api.resumeEffect(arg));
  }

  @override
  Future<IntValue> setAudioEffectEventCallback() {
    return wrapper('setAudioEffectEventCallback', _api.setAudioEffectEventCallback());
  }

  @override
  Future<IntValue> setEffectPlaybackVolume(SetEffectPlaybackVolumeRequest arg) {
    return wrapper('setEffectPlaybackVolume', _api.setEffectPlaybackVolume(arg));
  }

  @override
  Future<IntValue> setEffectSendVolume(SetEffectSendVolumeRequest arg) {
    return wrapper('setEffectSendVolume', _api.setEffectSendVolume(arg));
  }

  @override
  Future<IntValue> stopAllEffects() {
    return wrapper('stopAllEffects', _api.stopAllEffects());
  }

  @override
  Future<IntValue> stopEffect(IntValue arg) {
    return wrapper('stopEffect', _api.stopEffect(arg));
  }
}